package jalview.ext.ensembl;

import jalview.datamodel.AlignmentI;

import java.io.BufferedReader;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Arrays;
import java.util.List;

import org.json.simple.JSONObject;
import org.json.simple.parser.JSONParser;
import org.json.simple.parser.ParseException;

public class EnsemblLookup extends EnsemblRestClient
{

  @Override
  public String getDbName()
  {
    return "ENSEMBL";
  }

  @Override
  public AlignmentI getSequenceRecords(String queries) throws Exception
  {
    return null;
  }

  @Override
  protected URL getUrl(List<String> ids) throws MalformedURLException
  {
    String identifier = ids.get(0);
    return getUrl(identifier);
  }

  /**
   * @param identifier
   * @return
   */
  protected URL getUrl(String identifier)
  {
    String url = ENSEMBL_REST + "/lookup/id/" + identifier
            + "?content-type=application/json";
    try
    {
      return new URL(url);
    } catch (MalformedURLException e)
    {
      return null;
    }
  }

  @Override
  protected boolean useGetRequest()
  {
    return true;
  }

  @Override
  protected String getRequestMimeType(boolean multipleIds)
  {
    return "application/json";
  }

  @Override
  protected String getResponseMimeType()
  {
    return "application/json";
  }

  /**
   * Calls the Ensembl lookup REST endpoint and retrieves the 'Parent' for the
   * given identifier, or null if not found
   * 
   * @param identifier
   * @return
   */
  public String getParent(String identifier)
  {
    List<String> ids = Arrays.asList(new String[] { identifier });
  
    BufferedReader br = null;
    try
    {
      URL url = getUrl(identifier);
      if (url != null)
      {
        br = getHttpResponse(url, ids);
      }
      return (parseResponse(br));
    } catch (IOException e)
    {
      // ignore
      return null;
    } finally
    {
      if (br != null)
      {
        try
        {
          br.close();
        } catch (IOException e)
        {
          // ignore
        }
      }
    }
  }

  /**
   * Parses "Parent" from the JSON response and returns the value, or null if
   * not found
   * 
   * @param br
   * @return
   * @throws IOException
   */
  protected String parseResponse(BufferedReader br) throws IOException
  {
    String parent = null;
    JSONParser jp = new JSONParser();
    try
    {
      JSONObject val = (JSONObject) jp.parse(br);
      parent = val.get("Parent").toString();
    } catch (ParseException e)
    {
      // ignore
    }
    return parent;
  }

}
